/*
 * Copyright (c) 2010 JAVART Sp. z o.o.
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Created by jzadrag on 28-10-2010
 * 
 */

/**
 * EventHub.java
 */
package org.volverine.server.event;

import java.net.SocketAddress;
import java.util.ArrayList;
import java.util.Iterator;
//import java.util.LinkedList;
import java.util.WeakHashMap;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * PooledEventBus is an event processing service that has it's own thread pool
 * 
 * @author jzadrag
 *
 */
public class PooledDataProcessor implements DataEventProcessor {
	private final WeakHashMap<SocketAddress,ArrayList<DataEventHandler>> registry = new WeakHashMap<SocketAddress,ArrayList<DataEventHandler>>();
	private final ExecutorService pool;
	
	private class EventProcessor implements Runnable {
		private final DataEvent event;
		
		protected EventProcessor(DataEvent event) {
			this.event=event;
		}
		
		public void run() {
			ArrayList<DataEventHandler> list = null;
			synchronized(registry) {
				list = registry.get(event.getClass());
			}
			if(list!=null) {
				for(DataEventHandler handler: list) {
					handler.handle(event);
					if(event instanceof Consumable) {
						if(((Consumable)event).isConsumed()) break;
					}
				}
			}
		}
	}
	
	public PooledDataProcessor(int noOfWorkerThreads) {
		 pool = Executors.newFixedThreadPool(noOfWorkerThreads);
	}

	/* (non-Javadoc)
	 * @see com.synergive.stealth.event.EventBus#addEventHandler(com.synergive.stealth.event.EventHandler)
	 */
	@Override
	public void addEventHandler(DataEventHandler handler) {
		synchronized(registry) {
			if(handler!=null) {
				SocketAddress key = handler.getSocketAddress();
				ArrayList<DataEventHandler> list;
				if(registry.containsKey(key)) { 
					list = registry.get(key);
				} else { // add to registry
					list = new ArrayList<DataEventHandler>();
					registry.put(key, list);
				}
				list.add(handler);
			}
		}
	}
	
	/* (non-Javadoc)
	 * @see com.synergive.stealth.event.EventBus#removeEventHandler(com.synergive.stealth.event.EventHandler)
	 */
	@Override
	public void removeEventHandler(DataEventHandler handler) {
		synchronized(registry) {
			if(handler!=null) {
				SocketAddress key = handler.getSocketAddress();
				if(registry.containsKey(key)) {
					ArrayList<DataEventHandler> list = registry.get(key);
					for(Iterator<DataEventHandler> i = list.iterator();i.hasNext();) {
						DataEventHandler data = i.next();
						if(data==handler) {
							i.remove();
							break;
						}
					}
					if(list.isEmpty()) registry.remove(key);
				}
			}
		}
	}
	
	public void postDataEvent(DataEvent event) {
		if(event!=null) {
			pool.submit(new EventProcessor(event));
		}
	}

}
